"""
(c) 2023 «Dacom M» Ltd
"""

from veilrpc.utils.bazalt import states_response, get_output, CommandExecutionError


@states_response
def attach_docker_container(container_name, bridge, ipaddress=None, gateway=None, mtu=None, macaddress=None):
    """
    Добавление интерфейсов к docker контейнеру и bridge.
    """

    cmd = f"ovs-docker add-port {bridge} eth1 {container_name}"
    if ipaddress:
        cmd += f" --ipaddress={ipaddress}"
    if gateway:
        cmd += f" --gateway={gateway}"
    if mtu is not None:
        cmd += f" --mtu={mtu}"
    if macaddress:
        cmd += f" --macaddress={macaddress}"
    cmd_ret = get_output(cmd)
    if not cmd_ret['result']:
        comment = f"An error occurred while attaching docker container: {cmd_ret['comment']}"
        raise CommandExecutionError(comment)
    return f"Docker container {container_name} successfully attached."


@states_response
def detach_docker_container(container_name, bridge):
    """
    Удаление интерфейсов у docker контейнера и bridge.
    """

    cmd = f"ovs-docker del-port {bridge} eth1 {container_name}"
    cmd_ret = get_output(cmd)
    if not cmd_ret['result']:
        comment = f"An error occurred while detaching docker container: {cmd_ret['comment']}"
        raise CommandExecutionError(comment)
    return f"Docker container {container_name} successfully attached."


@states_response
def configure_docker_vlan(container_name, bridge, vlan_tag):
    """
    Настройка vlan для docker контейнера
    """

    cmd = f"ovs-docker set-vlan {bridge} eth1 {container_name} {vlan_tag}"
    cmd_ret = get_output(cmd)
    if not cmd_ret['result']:
        comment = f"An error occurred while configuring vlan for docker container: {cmd_ret['comment']}"
        raise CommandExecutionError(comment)
    return f"Docker container {container_name} vlan successfully configured."
