"""
(c) 2023 «Dacom M» Ltd
"""

import yaml

from veilrpc.utils.bazalt import CommandExecutionError, states_response, get_output, create_file, states_result,\
    DATA_PATH

EAML_KEYS_STR = "--pkcs7-private-key=/etc/puppetlabs/puppet/eyaml/private_key.pkcs7.pem " \
                "--pkcs7-public-key=/etc/puppetlabs/puppet/eyaml/public_key.pkcs7.pem"
PUPPET_COMMON_EYAML_PATH = f'{DATA_PATH}/common.eyaml'


@states_response
def puppet_get_param(file_path, module, param, **kwargs):
    _param = f'{module}::{param}'
    create_file(file_path)
    try:
        with open(file_path) as f:
            conf = yaml.safe_load(f)
            return conf[_param] if _param in conf.keys() else None
    except Exception as err:
        comment = f'Unable to read {file_path}: {err}'
        raise CommandExecutionError(comment) from None


@states_response
def puppet_modify_param(file_path, module, param, value, **kwargs):
    create_file(file_path)
    try:
        with open(file_path, 'r+') as f:
            conf = yaml.safe_load(f)
            if not isinstance(conf, dict):
                conf = {}
            if module:
                conf[f'{module}::{param}'] = value
            else:
                conf[f'{param}'] = value
            f.seek(0)
            f.write(yaml.dump(conf))
            f.truncate()
    except Exception as err:
        comment = f'Unable to write configuration: {err}'
        raise CommandExecutionError(comment) from None
    return f'Param {param} successfully modified'


@states_response
def puppet_remove_param(file_path, module, param, **kwargs):
    create_file(file_path)
    try:
        with open(file_path, 'r+') as f:
            conf = yaml.safe_load(f)
            conf.pop(f'{module}::{param}')
            f.seek(0)
            f.write(yaml.dump(conf))
            f.truncate()
    except KeyError:
        pass
    except Exception as err:
        comment = f'Unable to modify configuration: {err}'
        raise CommandExecutionError(comment) from None
    return f'Param {param} successfully removed'


@states_response
def puppet_eyaml_decrypt(value, **kwargs):
    ret = get_output(f"eyaml decrypt {EAML_KEYS_STR} -s '{value}'", **kwargs)
    if ret['result']:
        return ret['comment'].strip()
    return None


@states_response
def puppet_eyaml_encrypt(key, value, **kwargs):
    res = states_result(get_output,
                        cmd=f"eyaml encrypt {EAML_KEYS_STR} -l '{key}' -s {value} --quiet --output=string",
                        **kwargs)
    _, hash_value = res.split(":")
    hash_value = hash_value.strip()
    return hash_value


@states_response
def puppet_eyaml_common(key, value, **kwargs) -> bool:
    ret = puppet_eyaml_encrypt(key, value)
    if ret['result']:
        ret_2 = puppet_modify_param(PUPPET_COMMON_EYAML_PATH,
                                    "", key, ret['comment'])
        if ret_2['result']:
            return True
    return False
