"""
(c) 2023 «Dacom M» Ltd
"""

import os
from veilrpc.utils.bazalt import states_response, CommandExecutionError,\
    get_output, states_result, LOCAL_MODULES_PATH


TIMEZONE_PATH = f'{LOCAL_MODULES_PATH}/timezone.yaml'


@states_response
def timedatectl():
    return states_result(get_output, cmd='timedatectl')


@states_response
def set_timezone(timezone):
    res = states_result(get_output, cmd=f'timedatectl set-timezone {timezone}')
    try:
        with open(TIMEZONE_PATH, 'w') as config:
            conf = f"timezone::timezone: {timezone}\n"
            config.write(conf)
    except Exception as err:
        comment = f"Unable to write timezone configuration: {err}"
        raise CommandExecutionError(comment) from None
    return res


@states_response
def etc_timezone():
    if os.path.exists('/etc/timezone'):
        with open('/etc/timezone', 'r') as f:
            return f.read().strip()
    else:
        if os.path.exists('/etc/localtime'):
            try:
                with open('/etc/localtime', 'r') as f:
                    return f.read().splitlines()[-1].strip()
            except UnicodeError:
                with open('/etc/localtime', 'rb') as f:
                    return f.read().splitlines()[-1].decode("utf-8").strip()
        else:
            return "GMT0"


@states_response
def zone_code():
    return states_result(get_output, cmd='date +%Z')


@states_response
def zone_offset():
    return states_result(get_output, cmd='date +%z')


@states_response
def zone_code_offset():
    res = states_result(get_output, cmd='date +%z,%Z')
    offset, code = res.strip().split(',')
    return dict(code=code, offset=offset)


@states_response
def set_zone(timezone):
    if os.path.exists('/etc/timezone'):
        with open('/etc/timezone', 'w') as f:
            f.write(timezone.strip())
            f.write('\n')


@states_response
def hwclock():
    res = states_result(get_output, cmd='timedatectl')
    for line in (x.strip() for x in res.splitlines()):
        if 'rtc in local tz' in line.lower():
            try:
                if line.split(':')[-1].strip().lower() == 'yes':
                    return 'localtime'
                else:
                    return 'UTC'
            except IndexError:
                pass
    msg = f'Failed to parse timedatectl output: {res}'
    raise CommandExecutionError(msg)


@states_response
def set_hwclock(clock):
    """Sets the hardware clock to be either UTC or localtime"""

    timezone = zone_code()['comment']

    zonepath = f'/usr/share/zoneinfo/{timezone}'

    if not os.path.exists(zonepath):
        raise CommandExecutionError(f'Zone \'{zonepath}\' does not exist')

    os.unlink('/etc/localtime')
    os.symlink(zonepath, '/etc/localtime')
    cmd_pattern = "sed -i 's/{}/{}/g' {}"
    if clock == 'UTC':
        cmd = cmd_pattern.format('^UTC=.*', 'UTC=yes', '/etc/default/rcS')
    elif clock == 'localtime':
        cmd = cmd_pattern.format('^UTC=.*', 'UTC=no', '/etc/default/rcS')
    else:
        raise CommandExecutionError('Invalid clock')
    cmd_ret = get_output(cmd)
    if not cmd_ret['result']:
        raise CommandExecutionError(cmd_ret['comment'])
    return 'Successfully set hardware clock'


@states_response
def ntp_test_connection(ntp_server, **kwargs):
    return states_result(get_output, cmd=f'ntpdate -q {ntp_server}')


@states_response
def ntp_state(**kwargs):
    return states_result(get_output, cmd='ntpq -pn')
