"""
(c) 2023 «Dacom M» Ltd
"""

import locale
import pytz


VALID_LOCALES = sorted(set(locale.locale_alias.values()))
VALID_KEYBOARD_LAYOUTS = [
    "af", "al", "am", "ara", "at", "az", "ba", "bd", "be", "bg", "br", "brai",
    "bt", "bw", "by", "ca", "cd", "ch", "cm", "cn", "cz", "de", "dk", "ee",
    "epo", "es", "et", "fi", "fo", "fr", "gb", "ge", "gh", "gn", "gr", "hr",
    "hu", "ie", "il", "in", "iq", "ir", "is", "it", "jp", "ke", "kg", "kh",
    "kr", "kz", "la", "latam", "lk", "lt", "lv", "ma", "mao", "md", "me", "mk",
    "ml", "mm", "mn", "mt", "mv", "nec_vndr/jp", "ng", "nl", "no", "np", "ph",
    "pk", "pl", "pt", "ro", "rs", "ru", "se", "si", "sk", "sn", "sy", "th",
    "tj", "tm", "tr", "tw", "tz", "ua", "us", "uz", "vn", "za"]
VALID_TIMEZONES = sorted(pytz.common_timezones)
VIDEO_TYPES = ["vga", "cirrus", "virtio", "qxl", "none"]
SOUND_TYPES = ["es1370", "sb16", "ac97", "ich6", "usb"]
WATCHDOG_MODELS = ["i6300esb", "ib700"]
WATCHDOG_ACTIONS = ["reset", "shutdown", "poweroff", "pause", "none", "dump", "inject-nmi"]
DEFAULT_WATCHDOG = {'enabled': False}
# allowed_models = ['piix3-uhci', 'piix4-uhci', 'ehci', 'ich9-ehci1', 'ich9-uhci1',
#                  'ich9-uhci2', 'ich9-uhci3', 'vt82c686b-uhci', 'pci-ohci',
#                  'nec-xhci', 'qusb1', 'qusb2', 'qemu-xhci', 'none']
# USBv1, USBv1, USBv2, USBv3(default)
USB_CONTROLLER_TYPES = ['piix3-uhci', 'pci-ohci', 'ehci', 'nec-xhci']
CONTROLLERS_TYPES = ['pci', 'ide', 'scsi', 'sata', 'usb']
DEFAULT_FEATURES = ['acpi', 'apic']
ENABLE_FEATURES = {'pae': '<pae/>',
                   'hap_on': "<hap state='on'/>",
                   'hap_off': "<hap state='off'/>",
                   'pvspinlock_on': "<pvspinlock state='on'/>",
                   'pvspinlock_off': "<pvspinlock state='off'/>",
                   'kvm_hidden_on': "<kvm>\n<hidden state='on'/>\n</kvm>",
                   'kvm_hidden_off': "<kvm>\n<hidden state='off'/>\n</kvm>",
                   'pmu_on': "<pmu state='on'/>",
                   'pmu_off': "<pmu state='off'/>"}
DEFAULT_ARCH = "x86_64"
ARCH_TYPES = [DEFAULT_ARCH, 'e2k']
DEFAULT_MACHINE = "pc"
MACHINE_TYPES = [DEFAULT_MACHINE, 'q35', 'e2k']
CONTROLLERS_PC_SLOTS = {'pci': 31, 'ide': 4, 'scsi': 6, 'sata': 6, 'usb': 4}
CONTROLLERS_Q35_SLOTS = {'pci': 93, 'ide': 0, 'scsi': 6, 'sata': 6, 'usb': 4}
# Список контроллеров создаваемых на ВМ по умолчанию.
# Запись вида 'ide': [{}] — означает, что создаётся один ide контроллер без аргументов.
# Запись вида 'scsi': [] — означает, что scsi контроллеров добавлено не будет.
DEFAULT_CONTROLLER_PC_CONFIG = {
    'pci': [{'model': 'pci-root', 'order': 0},
            {'model': 'pci-bridge', 'order': 1}],
    'scsi': [],
    'ide': [{}],
    'sata': [{}],
    'usb': [{'model': 'nec-xhci', 'order': 0}]
}
DEFAULT_CONTROLLER_Q35_CONFIG = {
    'pci': [{'model': 'pcie-root', 'order': 0},
            {'model': 'pcie-root-port', 'order': 1},
            {'model': 'pcie-root-port', 'order': 2},
            {'model': 'pcie-root-port', 'order': 3},
            {'model': 'pcie-root-port', 'order': 4},
            {'model': 'pcie-root-port', 'order': 5},
            {'model': 'pcie-root-port', 'order': 6},
            {'model': 'pcie-root-port', 'order': 7},
            {'model': 'pcie-to-pci-bridge', 'order': 8}],
    'scsi': [],
    'ide': [],
    'sata': [{}],
    'usb': [{'model': 'nec-xhci', 'order': 0}]
}
DEFAULT_VIDEO_DEVICE_CONFIG = {'type': 'cirrus', 'vram': 16384, 'heads': 1}
DEFAULT_LINUX_VIDEO_DEVICE_CONFIG = {'type': 'virtio', 'vram': 16384, 'heads': 1}
DEFAULT_OTHER_VIDEO_DEVICE_CONFIG = {'type': 'vga', 'vram': 16384, 'heads': 1}
NONE_VIDEO_DEVICE_CONFIG = {'type': 'none', 'vram': 16384, 'heads': 1}
DEFAULT_SOUND_DEVICE_CONFIG = {'enabled': True, 'model': 'ich6', 'codec': 'micro'}
# Шины, к которым возможен hot plug устройств
HOT_PLUGGABLE_BUSES = ['pci', 'scsi']
CPU_MODES = ["default", "host-model", "host-passthrough", "custom"]
CPU_SHARES = 1024
CPU_PERIOD = 100000
CPU_QUOTA = CPU_GLOBAL_QUOTA = CPU_EMULATOR_QUOTA = 17592186044415
VCPU_PER_CORE = 4
MEMORY_SHARES = 1024
BOOTMENU_TIMEOUT = 3
DEFAULT_MEMORY_UNIT = 'MiB'
MIN_MEMORY_COUNT = 50
CHOICES_GRAPHICS_SETTINGS = {
    'image_compression': ['auto_glz', 'auto_lz', 'quic', 'glz', 'lz', 'off'],
    'jpeg_compression': ['auto', 'never', 'always'],
    'zlib_compression': ['auto', 'never', 'always'],
    'playback_compression': ['on', 'off'],
    'streaming_mode': ['off', 'all', 'filter'],
    'clipboard_copypaste': ['yes', 'no'],
    'filetransfer_enable': ['yes', 'no'],
    'mouse_mode': ['client', 'server']
}
