"""
(c) 2023 «Dacom M» Ltd
"""

from collections import namedtuple


class NicDriver:
    virtio = 'virtio'
    e1000 = 'e1000'
    rtl8139 = 'rtl8139'
    vmxnet3 = 'vmxnet3'


OS_TYPES = namedtuple('OS', 'windows linux other')(windows='Windows', linux='Linux', other='Other')

OS_TEMPLATES = {
    OS_TYPES.windows: {'controllers':
                           {'vdisk': {'target_bus': 'sata'},  # noqa: E127
                            'vmachineinf': {'nic_driver': NicDriver.e1000}
                            },
                       'video':
                           {'type': 'cirrus'},
                       'sound':
                           {'model': 'ich6', 'codec': 'micro'},
                       'os_version': {'dos': 'Microsoft MS-DOS'}
                       },
    OS_TYPES.linux: {'controllers':
                         {'vdisk': {'target_bus': 'virtio'},  # noqa: E127
                          'vmachineinf': {'nic_driver': NicDriver.virtio}
                          },
                     'video':
                         {'type': 'virtio'},
                     'sound':
                         {'model': 'ich6', 'codec': 'micro'},
                     'os_version': {'linux64': 'Other Linux (64-bit)'}
                     },
    OS_TYPES.other: {'controllers':
                         {'vdisk': {'target_bus': None},
                          'vmachineinf': {'nic_driver': None}
                          },
                     'video':
                         {'type': 'vga'},
                     'sound':
                         {'model': 'ich6', 'codec': 'micro'},
                     'os_version': {'unknown': 'UNKNOWN OS'}
                     }
}

OS_VERSIONS = {
    OS_TYPES.windows: {'dos': 'Microsoft MS-DOS',
                       'windows31': 'Microsoft Windows 3.1',
                       'windows95': 'Microsoft Windows 95',
                       'windows98': 'Microsoft Windows 98',
                       'windowsNT': 'Microsoft Windows NT',
                       'windows2000': 'Microsoft Windows 2000',
                       'windowsXP32': 'Microsoft Windows XP Professional (32-bit)',
                       'windowsXP64': 'Microsoft Windows XP Professional (64-bit)',
                       'windowsVistax32': 'Microsoft Windows Vista (32-bit)',
                       'windowsVistax64': 'Microsoft Windows Vista (64-bit)',
                       'windows7x32': 'Microsoft Windows 7 (32-bit)',
                       'windows7x64': 'Microsoft Windows 7 (64-bit)',
                       'windows8x32': 'Microsoft Windows 8.x (32-bit)',
                       'windows8x64': 'Microsoft Windows 8.x (64-bit)',
                       'windows10x32': 'Microsoft Windows 10 (32-bit)',
                       'windows10x64': 'Microsoft Windows 10 (64-bit)',
                       'windows11x64': 'Microsoft Windows 11 (64-bit)',
                       'windowsS2003x32': 'Microsoft Windows Server 2003 (32-bit)',
                       'windowsS2003x64': 'Microsoft Windows Server 2003 (64-bit)',
                       'windowsS2008x32': 'Microsoft Windows Server 2008 (32-bit)',
                       'windowsS2008x64': 'Microsoft Windows Server 2008 (64-bit)',
                       'windowsS2008R2x64': 'Microsoft Windows Server 2008 R2 (64-bit)',
                       'windowsS2012x64': 'Microsoft Windows Server 2012 (64-bit)',
                       'windowsS2016x64': 'Microsoft Windows Server 2016 (64-bit)',
                       'windowsS2019x64': 'Microsoft Windows Server 2019 (64-bit)',
                       'windowsS2022x64': 'Microsoft Windows Server 2022 (64-bit)',
                       },
    OS_TYPES.linux: {'linux32': 'Other Linux (32-bit)',
                     'linux64': 'Other Linux (64-bit)',
                     'linux24x32': 'Other 2.4.x Linux (32-bit)',
                     'linux24x64': 'Other 2.4.x Linux (64-bit)',
                     'linux26x32': 'Other 2.6.x Linux (32-bit)',
                     'linux26x64': 'Other 2.6.x Linux (64-bit)',
                     'linux3x32': 'Other 3.x Linux (32-bit)',
                     'linux3x64': 'Other 3.x Linux (64-bit)',
                     'linux4x32': 'Other 4.x Linux (32-bit)',
                     'linux4x64': 'Other 4.x Linux (64-bit)',
                     'chromeos': 'Chrome OS Linux (64-bit)',
                     'coreos': 'Core OS Linux (64-bit)',
                     'ubuntu32': 'Ubuntu Linux (32-bit)',
                     'ubuntu64': 'Ubuntu Linux (64-bit)',
                     'oracle4x32': 'Oracle Linux 4 (32-bit)',
                     'oracle4x64': 'Oracle Linux 4 (64-bit)',
                     'oracle5x32': 'Oracle Linux 5 (32-bit)',
                     'oracle5x64': 'Oracle Linux 5 (64-bit)',
                     'oracle6x32': 'Oracle Linux 6 (32-bit)',
                     'oracle6x64': 'Oracle Linux 6 (64-bit)',
                     'oracle7x64': 'Oracle Linux 7 (64-bit)',
                     'oracle8x64': 'Oracle Linux 8 (64-bit)',
                     'fedora32': 'Red Hat Fedora (32-bit)',
                     'fedora64': 'Red Hat Fedora (64-bit)',
                     'debian4x32': 'Debian GNU/Linux 4 (32-bit)',
                     'debian4x64': 'Debian GNU/Linux 4 (64-bit)',
                     'debian5x32': 'Debian GNU/Linux 5 (32-bit)',
                     'debian5x64': 'Debian GNU/Linux 5 (64-bit)',
                     'debian6x32': 'Debian GNU/Linux 6 (32-bit)',
                     'debian6x64': 'Debian GNU/Linux 6 (64-bit)',
                     'debian7x32': 'Debian GNU/Linux 7 (32-bit)',
                     'debian7x64': 'Debian GNU/Linux 7 (64-bit)',
                     'debian8x32': 'Debian GNU/Linux 8 (32-bit)',
                     'debian8x64': 'Debian GNU/Linux 8 (64-bit)',
                     'debian9x32': 'Debian GNU/Linux 9 (32-bit)',
                     'debian9x64': 'Debian GNU/Linux 9 (64-bit)',
                     'debian10x32': 'Debian GNU/Linux 10 (32-bit)',
                     'debian10x64': 'Debian GNU/Linux 10 (64-bit)',
                     'debian11x32': 'Debian GNU/Linux 11 (32-bit)',
                     'debian11x64': 'Debian GNU/Linux 11 (64-bit)',
                     'spacex64': 'SpaceVM (64-bit)',
                     'alpine': 'Alpine (64-bit)',
                     'alt9x64': 'Alt Linux 9 (64-bit)',
                     'alt10x64': 'Alt Linux 10 (64-bit)',
                     'alteros': 'AlterOS (64-bit)',
                     'astrasmolenskx64': 'Astra Smolensk (64-bit)',
                     'astraorelx64': 'Astra Orel (64-bit)',
                     'el6x64': 'Elbrus OS 6 (64-bit)',
                     'redos7x64': 'РЕД ОС (64-bit)',
                     'rosax64': 'ROSA (64-bit)',
                     'rosacobalt': 'ROSA Cobalt (64-bit)',
                     'centos4x32': 'CentOS 4 (32-bit)',
                     'centos4x64': 'CentOS 4 (64-bit)',
                     'centos5x32': 'CentOS 5 (32-bit)',
                     'centos5x64': 'CentOS 5 (64-bit)',
                     'centos6x32': 'CentOS 6 (32-bit)',
                     'centos6x64': 'CentOS 6 (64-bit)',
                     'centos7x64': 'CentOS 7 (64-bit)',
                     'centos8x64': 'CentOS 8 (64-bit)',
                     'centosstream': 'CentOS Stream (64-bit)',
                     'suse8x32': 'SUSE Enterprise 8 (32-bit)',
                     'suse8x64': 'SUSE Enterprise 8 (64-bit)',
                     'suse9x32': 'SUSE Enterprise 9 (32-bit)',
                     'suse9x64': 'SUSE Enterprise 9 (64-bit)',
                     'suse10x32': 'SUSE Enterprise 10 (32-bit)',
                     'suse10x64': 'SUSE Enterprise 10 (64-bit)',
                     'suse11x32': 'SUSE Enterprise 11 (32-bit)',
                     'suse11x64': 'SUSE Enterprise 11 (64-bit)',
                     'suse12x64': 'SUSE Enterprise 12 (64-bit)',
                     'suse15x64': 'SUSE Enterprise 15 (64-bit)',
                     'redhat21': 'Red Hat Enterprise Linux 2.1',
                     'redhat3x32': 'Red Hat Enterprise 3 (32-bit)',
                     'redhat3x64': 'Red Hat Enterprise 3 (64-bit)',
                     'redhat4x32': 'Red Hat Enterprise 4 (32-bit)',
                     'redhat5x64': 'Red Hat Enterprise 5 (64-bit)',
                     'redhat6x32': 'Red Hat Enterprise 6 (32-bit)',
                     'redhat6x64': 'Red Hat Enterprise 6 (64-bit)',
                     'redhat7x64': 'Red Hat Enterprise 7 (64-bit)',
                     'redhat8x64': 'Red Hat Enterprise 8 (64-bit)',
                     'redhat9x64': 'Red Hat Enterprise 9 (64-bit)',
                     'photon': 'WMware Photon OS (64-bit)'
                     },
    OS_TYPES.other: {'unknown': 'UNKNOWN OS',  # Базовое значение, не трогать
                     'ibm': 'IBM OS/2',
                     'otherx32': 'Other (32-bit)',
                     'otherx64': 'Other (64-bit)',
                     'macos106x32': 'Apple Mac OS X 10.6 (32-bit)',
                     'macos106x64': 'Apple Mac OS X 10.6 (64-bit)',
                     'macos107x32': 'Apple Mac OS X 10.7 (32-bit)',
                     'macos107x64': 'Apple Mac OS X 10.7 (64-bit)',
                     'macos108x64': 'Apple Mac OS X 10.8 (64-bit)',
                     'macos109x64': 'Apple Mac OS X 10.9 (64-bit)',
                     'macos1010x64': 'Apple Mac OS X 10.10 (64-bit)',
                     'macos1011x64': 'Apple Mac OS X 10.11 (64-bit)',
                     'macos1012x64': 'Apple Mac OS X 10.12 (64-bit)',
                     'macos1013x64': 'Apple Mac OS X 10.13 (64-bit)',
                     'macos1014x64': 'Apple Mac OS X 10.14 (64-bit)',
                     'macos1015x64': 'Apple Mac OS X 10.15 (64-bit)',
                     'macos11x64': 'Apple Mac OS X 11 (64-bit)',
                     'macos12x64': 'Apple Mac OS X 12 (64-bit)',
                     'freebsd11x64': 'FreeBSD 11 (64-bit)',
                     'freebsd12x64': 'FreeBSD 12 (64-bit)',
                     'freebsd13x64': 'FreeBSD 13 (64-bit)',
                     'novell51': 'Novell NetWare5.1',
                     'novell6x': 'Novell NetWare6.x',
                     'solaris8': 'Sun Microsystems Solaris 8',
                     'solaris9': 'Sun Microsystems Solaris 9',
                     'solaris10x32': 'Oracle Solaris 10(32-bit)',
                     'solaris10x64': 'Oracle Solaris 10(64-bit)',
                     'solaris11x64': 'Oracle Solaris 11(64-bit)',
                     'vmware4x': 'VMware ESXi 4.x',
                     'vmware5x': 'VMware ESXi 5.x',
                     'vmware6x': 'VMware ESXi 6.x',
                     'vmware7x': 'VMware ESXi 7.x',
                     'ecom1': 'Serenity Systems eComStation 1',
                     'ecom2': 'Serenity Systems eComStation 2',
                     'unixware7': 'SCO UnixWare 7',
                     'openserver5': 'SCO OpenServer 5',
                     'openserver6': 'SCO OpenServer 6',
                     }
}

linux_packet_managers = {
    'apt-get': [
        'ubuntu32',
        'ubuntu64',
        'debian4x32',
        'debian4x64',
        'debian5x32',
        'debian5x64',
        'debian6x32',
        'debian6x64',
        'debian7x32',
        'debian7x64',
        'debian8x32',
        'debian8x64',
        'debian9x32',
        'debian9x64',
        'debian10x32',
        'debian10x64',
        'debian11x32',
        'debian11x64',
        'veilx64',
        'alt9x64',
        'alt10x64',
        'astrasmolenskx64',
        'astraorelx64',
    ],
    'yum': [
        'fedora32',
        'fedora64',
        'redos7x64',
        'rosax64',
        'rosacobalt',
        'centos4x32',
        'centos4x64',
        'centos5x32',
        'centos5x64',
        'centos6x32',
        'centos6x64',
        'centos7x64',
        'centos8x64',
        'centosstream',
        'redhat21',
        'redhat3x32',
        'redhat3x64',
        'redhat4x32',
        'redhat5x64',
        'redhat6x32',
        'redhat6x64',
        'redhat7x64',
        'redhat8x64',
        'redhat9x64',
    ],
}


def get_os_version_id(os_type, os_version_name) -> str:
    for k, v in OS_VERSIONS[os_type].items():
        if v == os_version_name:
            return k
    return 'unknown'
